//----------------------------------------------------------------------------------
//	FILE:			BlinkingLED-Main.C
//
//	Description:	This program blinks LD2 on the Piccolo controlSTICK at a 
//					frequency given by the CPU timer period register.  Change the 
//					register/bits CpuTimer0Regs.PRD.all to change the frequency of 
//					the LED (LD2).
//
//  Target:  		TMS320F2802x or TMS320F2803x families (Piccolo)
//
//----------------------------------------------------------------------------------
//  $TI Release:$ 	V1.1
//  $Release Date:$ 26 Oct 2009 - BRL
//----------------------------------------------------------------------------------
//
// PLEASE READ - Useful notes about this Project

// Although this project is made up of several files, the most important ones are:
//	 "BlinkingLED-Main.c"	- this file
//		- Application Initialization, Peripheral config
//		- Application management
//		- Slower background code loops and Task scheduling
//	 "BlinkingLED-DevInit_F28xxx.c"
//		- Device Initialization, e.g. Clock, PLL, WD, GPIO mapping
//		- Peripheral clock enables
// The other files are generally used for support and defining the registers as C
// structs. In general these files will not need to be changed.
//   "F28027_RAM_BlinkingLED.CMD" or "F28027_FLASH_BlinkingLED.CMD"
//		- Allocates the program and data spaces into the device's memory map.
//   "DSP2802x_Headers_nonBIOS.cmd" and "DSP2802x_GlobalVariableDefs.c"
//		- Allocate the register structs into data memory.  These register structs are
//		  defined in the peripheral header includes (DSP2802x_Adc.h, ...) 
//
//----------------------------------------------------------------------------------

#include "PeripheralHeaderIncludes.h"
#include "DSP28x_Project.h"
#include "DSP2802x_EPwm_defines.h"

#ifndef DSP2802x_EPWM_DEFINES_H
	#include "DSP2802x_EPwm_defines.h"
#endif

#ifndef DSP2802x_DEVICE_H
	#include "PeripheralHeaderIncludes.h"
#endif

// PWM Frequency Definition
#define SYSCLK_FREQ_MHZ       60                                     // PWM Clock set to SYSCLKOUT = 60MHz
#define PWM_FREQ_KHZ          200
#define PWM_PERIOD_CNTS       ((Uint32)SYSCLK_FREQ_MHZ*1000 / PWM_FREQ_KHZ)
// ADC Triggering via ePWM's CMPB
#define ADC_CNTS_PER_uS       SYSCLK_FREQ_MHZ
#define MICSEC_TO_NS          1000                // 1000ns / 1us
#define ADC_TRIGGER_TIME_NS   200                 // Trigger ADC 200ns after PWM counter resets to zero
#define ADC_TRIG_CNTS         (ADC_TRIGGER_TIME_NS*ADC_CNTS_PER_uS / MICSEC_TO_NS)  // Be careful of truncation error
// Register constants
#define DUTY_COARSE           (EPwm2Regs.CMPAM.half.CMPA)          // Set PWM Duty Cycle with coarse resolution (uses only the ePWM register)
#define DUTY_FINE             (EPwm2Regs.CMPAM.all)                // Set PWM Duty Cycle with fine resolution (uses ePWM register AND HRPWM 8-bit extension)
#define ADC_TRIGGER           (EPwm2Regs.CMPB)                     // ePWM's 16-bit CMPB register being used to trigger ADC SOC
// PWM Duty Cycle Limits for normal operation
#define MAX_PWM_CNT_COARSE    ((PWM_PERIOD_CNTS*90 + 50) / 100)           // 90% PWM
#define MIN_PWM_CNT_COARSE    ((PWM_PERIOD_CNTS*10 + 50) / 100)           // 10% PWM
#define MAX_PWM_CNT_FINE      (((PWM_PERIOD_CNTS<<8)*90 + 50) / 100)      // 90% HRPWM
#define MIN_PWM_CNT_FINE      (((PWM_PERIOD_CNTS<<8)*10 + 50) / 100)      // 10% HRPWM


// PWM Macros
#define M_ForceDutyCycleZero()  (DUTY_FINE   = ( 0x000000 << 8 ))  // Bypasses any duty cycle limits of the other macros
#define M_SetPWMDutyCoarse(x)   (DUTY_COARSE = ( x & 0xFFFF ) )        // Limit to 16-bits (size of CMPA register)
#define M_SetPWMDutyFine(x)     (DUTY_FINE   = ( x & 0xFFFFFF) << 8 )  // Limit value sent to size of CMPA(16) + CMPAHR(8) = 24 bits, then Left Shift by 8 (LSByte is Reserved)
#define M_SetADCTrigger()       (ADC_TRIGGER = ( ADC_TRIG_CNTS & 0xFFFF ) )


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// FUNCTION PROTOTYPES
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void DeviceInit(void);
void InitFlash(void);
void MemCopy(Uint16 *SourceAddr, Uint16* SourceEndAddr, Uint16* DestAddr);

void InitADC(void);
void RunDutyCycleTest(void);
interrupt void adc_isr(void);
void InitPWM(void);
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// VARIABLE DECLARATIONS - GENERAL
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// Used for running BackGround in flash and the ISR in RAM
extern Uint16 RamfuncsLoadStart, RamfuncsLoadEnd, RamfuncsRunStart;

Uint16 m_ADCINA1 = 0;
Uint16 m_ADCINB7 = 0;
Uint16 m_ADCINA7 = 0;
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// MAIN CODE - starts here
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
void main(void)
{
	static Uint16 ToggleLEDCnt = 0;
//=================================
//	INITIALISATION - General
//=================================

	DeviceInit();	// Device Life support & GPIO mux settings

// Only used if running from FLASH
// Note that the variable FLASH is defined by the compiler (-d FLASH)
#ifdef FLASH
// Copy time critical code and Flash setup code to RAM
// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
// symbols are created by the linker. Refer to the linker files. 
	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);

// Call Flash Initialization to setup flash waitstates
// This function must reside in RAM
	InitFlash();	// Call the flash wrapper init function
#endif //(FLASH)

	// reassign ADC ISR after Pie Vector Table has been initialized via DeviceInit()
	EALLOW;  // This is needed to write to EALLOW protected register
//	   PieVectTable.ADCINT1 = &adc_isr;
	EDIS;    // This is needed to disable write to EALLOW protected registers


	// initialize PWM.  Initial duty cycle is 0%.
	InitPWM();

	// Initialize ADC
	InitADC();

	// Enable INT 10.3 in the PIE
	PieCtrlRegs.PIEIER10.bit.INTx3 = 1;  // Set PIE Interrupt Enable bit for INT group 10, bit 3 (corresponding to ADCINT3)

// Initialize Period of CPU Timers
// Timer period definitions found in PeripheralHeaderIncludes.h
	CpuTimer0Regs.PRD.all =  uSec145;	// 500ms * 2(# of LED states) = 1s blink rate
//	CpuTimer1Regs.PRD.all =  mSec1000;	
//	CpuTimer2Regs.PRD.all =  mSec5000;	


//=================================
//	INTERRUPT INITIALISATION (not needed in this example)
//  (best to run this section after other initialisation)
//=================================

// Enable Peripheral, global Ints and higher priority real-time debug events:
	IER |= M_INT10;
	EINT;   // Enable Global interrupt INTM
	ERTM;   // Enable Global realtime interrupt DBGM


//=================================
//	Forever LOOP
//=================================

	// Loop Forever
	for(;;)  //infinite loop
	{
		if(CpuTimer0Regs.TCR.bit.TIF == 1)
		{
			// clear flag
			CpuTimer0Regs.TCR.bit.TIF = 1;

			if( ToggleLEDCnt++ >= 3448 )
			{
				ToggleLEDCnt = 0;
				//Toggle GPIO34 (LD2, Red)
				GpioDataRegs.GPBTOGGLE.bit.GPIO34 = 1;
				// HW Test - straight 90% duty cycle = 69120 counts for HRPWM
		//		M_SetPWMDutyFine( 69120 );

// Uncomment to bypass ADC isr.
//				m_ADCINA1 = AdcResult.ADCRESULT0;
//				m_ADCINA7 = AdcResult.ADCRESULT1;
//				m_ADCINB7 = AdcResult.ADCRESULT2;
			}
			else
			{
				// HW Test - straight 20% duty cycle = 38400 counts for HRPWM
		//		M_SetPWMDutyFine( 15360 );
			}

			// vary duty cycle as a test
//			RunDutyCycleTest();
			M_SetPWMDutyFine( 38400 );
//			m_ADCINA1 = AdcResult.ADCRESULT0;
//			m_ADCINA7 = AdcResult.ADCRESULT1;
//			m_ADCINB7 = AdcResult.ADCRESULT2;

		}
	}
} //END MAIN CODE


void InitADC(void)
{
	Uint16 i = 0;

	EALLOW;
	AdcRegs.ADCCTL1.bit.ADCBGPWD	= 1;	// Power up band gap
	AdcRegs.ADCCTL1.bit.ADCREFPWD	= 1;	// Power up reference
	AdcRegs.ADCCTL1.bit.ADCPWDN 	= 1;	// Power up rest of ADC
	AdcRegs.ADCCTL1.bit.ADCENABLE	= 1;	// set timing to 13cycles as required by ADC module
	for(i=0; i<5000; i++){}				// wait 60000 cycles = 1ms (each iteration is 12 cycles)

	// Configure ADC
	AdcRegs.ADCCTL1.bit.ADCREFSEL     = 0;    // Use internal 3.3v reference
	AdcRegs.ADCCTL1.bit.INTPULSEPOS	  = 1;	  // create INT pulses 1 cycle prior to output latch (at end of conversion)
	AdcRegs.ADCCTL2.bit.ADCNONOVERLAP = 0;    // Allow overlap of conversion w/ next sample
	AdcRegs.ADCCTL2.bit.CLKDIV2EN     = 0;    // ADC Clk = CPU Clock = 60MHz here
	AdcRegs.SOCPRICTL.bit.SOCPRIORITY = 0x03; // Force priority to be (Highest->Lowest: SOC0->SOC2)
	// Use a single Trigger (ePWM2, SOCA) to cause simultaneous sample requests and let
	// the SOCPRIORITY setting determine the order.  ADCNONOVERLAP register bit is cleared to allow
	// sampling the next signal while the previous one gets converted.
	// SOC0 / EOC0 used for ADCINA1 (Vbat + Ibat*0.05)
	AdcRegs.ADCSOC0CTL.bit.CHSEL 	= 0x01; // set SOC0 channel select to ADCINA1
	AdcRegs.ADCSOC0CTL.bit.TRIGSEL 	= 0x07;	// set SOC0 start trigger on EPWM2, ADCSOCA
	AdcRegs.ADCSOC0CTL.bit.ACQPS 	= 0x06;	// set SOC0 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1) = 166.67ns at 60MHz ADC Clk
	// SOC1 / EOC1 used for ADCINA7 (Buck Vout)
	AdcRegs.ADCSOC1CTL.bit.CHSEL 	= 0x07; // set SOC1 channel select to ADCINA7
	AdcRegs.ADCSOC1CTL.bit.TRIGSEL 	= 0x07; // set SOC1 start trigger on EPWM2, ADCSOCA
	AdcRegs.ADCSOC1CTL.bit.ACQPS 	= 0x06; // set SOC1 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1) = 166.67ns at 60MHz ADC Clk
	// SOC2 / EOC2 used for ADCINB7 (Ibat)
	AdcRegs.ADCSOC2CTL.bit.CHSEL 	= 0x0F; // set SOC2 channel select to ADCINB7
	AdcRegs.ADCSOC2CTL.bit.TRIGSEL 	= 0x07; // set SOC2 start trigger on EPWM2, ADCSOCA
	AdcRegs.ADCSOC2CTL.bit.ACQPS 	= 0x06; // set SOC2 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1) = 166.67ns at 60MHz ADC Clk
	// Set up interrupts (see DSP2802x_DefaultISR.h for default ISR definitions)
	// Can monitor ADCINTFLG bits to make sure the conversions have occurred, but
	// should only need to actually enable interrupts for the last conversion (EOC2 -> ADCINB7 in this case)
	// to enter an ISR to get all data after all 3 conversions are complete.
	AdcRegs.INTSEL3N4.bit.INT3E     = 1;    // enable ADCINT3
	AdcRegs.INTSEL3N4.bit.INT3SEL   = 0x02; // Trigger ADCINT3 using EOC2 (per above, corresponding to ADCINB7 -> Ibat)
	AdcRegs.INTSEL3N4.bit.INT3CONT  = 0;    // 0: disable continuous sample mode
	AdcRegs.ADCINTFLGCLR.bit.ADCINT3 = 1;   // W1C - Clear Interrupt Flag
	EDIS;

	return;
}
void InitPWM(void)
{
	// Time-base registers
	EPwm2Regs.TBPRD = PWM_PERIOD_CNTS;               // Set timer period, PWM frequency = 1 / period
	EPwm2Regs.TBPHS.all = 0;			             // Time-Base Phase Register
	EPwm2Regs.TBCTR = 0;					         // Time-Base Counter Register - initialize to zero
	EPwm2Regs.TBCTL.bit.PRDLD = TB_IMMEDIATE;        // Set Immediate load for period
	EPwm2Regs.TBCTL.bit.CTRMODE = TB_COUNT_UP;       // Count-up mode: used for edge-aligned PWM
	EPwm2Regs.TBCTL.bit.PHSEN = TB_DISABLE;	         // Disable phase loading
	EPwm2Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_DISABLE;  // Disable Synchronization output signal
	EPwm2Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;         // Together HSPCLKDIV and CLKDIV form the PWM clock pre-scaler
	EPwm2Regs.TBCTL.bit.CLKDIV = TB_DIV1;            // as: TBCLK = SYSCLKOUT / (HSPCLKDIV * CLKDIV) = SYSCLKOUT = 60MHz, here

	// Setup shadow register load on ZERO
	EPwm2Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;      // CMPA register is in shadow register mode (double buffer)
	EPwm2Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;    // Load CMPA register from its shadow on CTR=Zero (PWM Rising Edge per below)

	// Set Duty Cycle (CMPA used.  See pwm.h macro and constant definitions)
	M_ForceDutyCycleZero();                          // Set duty cycle to 0% initially (This macro bypasses any minimum duty cycle constraints)
	// Set up SOCA to trigger ADC
	M_SetADCTrigger();                               // Set CMPB to trigger SOCA
	EPwm2Regs.ETSEL.bit.SOCAEN  = 1;                 // Enable ADC SOCA pulse
	EPwm2Regs.ETSEL.bit.SOCASEL = 6;                 // Generate SOCA pulse when time base counter = CMPB, when timer is incrementing
	EPwm2Regs.ETPS.bit.SOCAPRD  = ET_1ST;            // Generate SOCA pulse

	// Set actions
	EPwm2Regs.AQCTLA.bit.ZRO = AQ_SET;                // Set PWM2A on Zero event (rising PWM edge)
	EPwm2Regs.AQCTLA.bit.CAU = AQ_CLEAR;              // Clear PWM2A on Up-Count CompareA event (falling PWM edge)

	// Set HiRes registers
	EALLOW;
	EPwm2Regs.HRCNFG.all = 0x0;					     // Initialize HiRes PWM Control Register
	EPwm2Regs.HRCNFG.bit.EDGMODE = HR_FEP;           // Enable HiRes control on Falling edge
	EPwm2Regs.HRCNFG.bit.CTLMODE = HR_CMP;           // CMPAHR controls the MEP
	EPwm2Regs.HRCNFG.bit.HRLOAD  = HR_CTR_ZERO;      // Shadow load on CTR=Zero
	EDIS;


	return;
}


void RunDutyCycleTest(void)
{
	static Uint32 nTestDutyCycle = 0;

	// Increment duty cycle and wrap around.
	if( nTestDutyCycle++ > MAX_PWM_CNT_FINE )
	{
		nTestDutyCycle = MIN_PWM_CNT_FINE;
	}

	M_SetPWMDutyFine( nTestDutyCycle );

	return;
}


interrupt void adc_isr(void)
{
	m_ADCINA1 = AdcResult.ADCRESULT0;
	m_ADCINA7 = AdcResult.ADCRESULT1;
	m_ADCINB7 = AdcResult.ADCRESULT2;

	AdcRegs.ADCINTFLGCLR.bit.ADCINT3 = 1;  // W1C - Clear Interrupt Flag
//	PieCtrlRegs.PIEACK.bit.ACK10 = 1;      // W1C - Clear the PIE ACK bit for Group 10 to enable next PIE interrupt.

	return;
}
